!+
! Module: esensmap_mod
!
! Authors:  Hermann Brunner - AIP
!          
! Description: calculate global EPIC sensitivity map
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 

module esensmap_mod

  use dal
  use errorHandling
  use param  
  use edetect_io
  
  implicit none 
  
  private 

  public esensmap

contains 
  
  !---------------------------------------------------------------------
  subroutine esensmap
  !---------------------------------------------------------------------
    
    real :: ul, back,texp
    real, dimension(max_ins) :: xcen, ycen
    character(len=1024) :: ostring, fname
!    character(len=64), dimension(max_ima) :: image
    character(len=16), dimension(max_ima) :: instrume 
    character(len=1024), dimension(max_ima) :: c_bgima, c_expima, c_mask
    integer :: i, j, k, k0, k1, ix, iy, ix1, iy1, inst, ipoint
    real, dimension(3) :: xoff, yoff
    integer, dimension(2) :: npix
    integer, dimension(max_ima) :: band     
    real(kind=Single), dimension(:,:), pointer :: sens
    real(kind=Single), parameter :: psfrac = 0.3
       
    type(DataSetT) :: set,setO
    type(ArrayT) :: arr

    real :: gammp
         
    ul = realParameter("mlmin")
         
! input files

!   print *, "get bg_image"        
    call get_files("bkgimagesets",c_bgima, instrume, band,.true.)          
!   print *, "get expo_image"
    call get_files("expimagesets",c_expima, instrume, band,.false.)
!   print *, "get det_mask"
    call get_files("detmasksets",c_mask, instrume, band,.false.)

    xoff=0.
    yoff=0.
    inst=1

    do i = 1,num_instr
      do j = 1,num_image(i)
        ipoint=ip(i,j) 
      
        fname = c_bgima(ip(i,j))
        write(errstr,*) "Opening file ",fname
        call message(AppMsg,Verbose)
        call co_bgima(fname,npix,xoff,yoff,inst,ipoint,nim)

        fname = c_expima(ip(i,j))
        write(errstr,*) "Opening file ",fname
        call message(AppMsg,Verbose)
        call co_expima(fname,npix,xoff,yoff,inst,ipoint,nim)

      end do

      ipoint=ip(i,1)
      fname = c_mask(i) 
      write(errstr,*) "Opening file ",fname
      call message(AppMsg,Sparse)
      call co_mask(fname,npix,xoff,yoff,inst,ipoint)

      do j = 2,num_image(i)
        do iy = 1,npix(2)
          do ix = 1,npix(1)
            mask_image(ip(i,j),ix,iy)=mask_image(ipoint,ix,iy)
          end do
        end do
      end do   
 
    end do
    call message(AppMsg,Verbose,"Data read in; calulating sensitivity ...")

    ostring=stringParameter("sensimageset")
    setO=dataset(ostring,Create)
    set=dataset(c_bgima(1),Read)
        
    call cp_head(set,setO,"EPIC EXPOSURE SENSITIVITY MAP", 2)         
        
    call setAttribute(setO, "HDUCLASS", "OGIP    ", &
                            "Format conforms to OGIP/GSFC conventions")
    call setAttribute(setO, "HDUCLAS1", "IMAGE   ", &
                            "File contains an image")
    call setAttribute(setO, "HDUCLAS2", "GENERIC", &
                            "Sensitivity map")

    arr=addArray(setO,"PRIMARY",Real32,npix)
    sens => real32Array2Data(arr)

    do iy = 1,npix(2)
      do ix = 1,npix(1)

        back = 0.0
        texp = 0.0

        do ipoint = 1,nim
 
          do iy1 = iy-1,iy+1
            do ix1 = ix-1,ix+1
              if (ix1 >= 1 .and. ix1 <= npix(1) .and. &
                  iy1 >= 1 .and. iy1 <= npix(2)) then
                if (mask_image(ipoint,ix1,iy1)) then 
                  back = back + bg_image(ipoint,ix1,iy1)  
                  texp = texp + exp_image(ipoint,ix1,iy1)
                end if
              end if              
            end do
          end do

        end do

        if (texp > 0.0 .and. back > 0.0) then 

          k1=1
          do while (gammp(real(k1),back) <= ul)
            k1=k1*2
          end do
          k0=k1/2 
          do while (k1>k0+1)
            k=(k0+k1)/2
            if (gammp(real(k),back) > ul) then
              k1=k
            else
              k0=k
            end if
          end do       
       
          sens(ix-1,iy-1) = 9.0*nim/psfrac * (real(k1)-back)/texp
        else
          sens(ix-1,iy-1) = -1.0
        end if
        
      end do
    end do

! output of sensitivity image

    call release(arr) 
    call release(set)
    call release(setO) 

    call message(AppMsg,Verbose,"Done")

  end subroutine esensmap
  
end module esensmap_mod  




